use DFA::Cellular;
use Quantum::Superpositions;

my $dfa = DFA::Cellular->new(40,20,<<INIT);
****************************************
* 33                                   *
* 2  3 2                               *
* 1               2                    *
*   2                      4           *
* 2   2                2               *
*                                      *
*   3          2    2                  *
*                                      *
*                   2                  *
*          1               4           *
*                                      *
*                                      *
*       1  1     2    4                *
*                                      *
*    1                                 *
*                                      *
* 1                                    *
*    1                                 *
****************************************
INIT

$dfa->show;

sub next {
        my ($n) = @_;
        my $me   = $n->[1][1];
	my %state;
        if ($me eq '1') {
		$state{$n->[2][1] eq '*' ? 2 : $n->[1][2] eq '*' ? 4 : 0 } = 1;
        }
        if ($me eq '2') {
		$state{$n->[1][2] eq '*' ? 3 : $n->[0][1] eq '*' ? 1 : 0 } = 1;
        }
        if ($me eq '3') {
		$state{$n->[0][1] eq '*' ? 4 : $n->[1][0] eq '*' ? 2 : 0 } = 1;
        }
        if ($me eq '4') {
		$state{$n->[1][0] eq '*' ? 1 : $n->[2][1] eq '*' ? 3 : 0 } = 1;
        }
        $state{1} = 1 if $n->[0][0] eq 1;
        $state{2} = 1 if $n->[2][0] eq 2;
        $state{3} = 1 if $n->[2][2] eq 3;
        $state{4} = 1 if $n->[0][2] eq 4;
	delete $state{0};
        return ' ' unless keys %state;
	return any keys %state;
}

$dfa->addrule(
        DFA::Cellular::ProcRule->new(\&next)
);

$dfa->mapout( sub{my @states = eigenstates($_[0]);
		  @states==0 ? ' ' : @states>1 ? '@' : $_[0]} );


while (1) {
	$dfa->step;
	$dfa->show;
}
