use DFA::Cellular;
use Quantum::Superpositions;

sub fill {
	my ($width, $height, $molecules) = @_;
	my @cells = (" "x$width)x$height;
	for (1..$molecules) {
		my $x = rand($width-2)+1;
		my $y = rand($height-2)+1;
		substr($cells[$y],$x,1) = 1 + int rand 4;
	}
	$cells[0] = $cells[-1] = "*"x$width;
	substr($cells[$_],0,1) =
	  substr($cells[$_],$width/2,1) =
	    substr($cells[$_],-1,1) = '*'
		for 0..$height-1;
	substr($cells[$_],$width/2,1) = '|'
		for 4..$height-5;
	return ($width, $height, join("\n", @cells));
}

print "N=";
chomp(my $N = <>);
my $dfa = DFA::Cellular->new(fill(40,20,$N));

sub next {
        my ($n) = @_;
        my $me  = $n->[1][1];
	my %new;

	# WALLS
        return '*' if $me eq '*';

	# BOUNCE OR DIE
	$new{ $n->[2][1] eq '*' ? 2
	    : $n->[1][2] eq '*' || $n->[1][2] eq '|' ? 4 : 0}=1 if $me eq 1;
	$new{ $n->[1][2] eq '*' || $n->[1][2] eq '|' ? 3
	    : $n->[0][1] eq '*' ? 1 : 0}=1 if $me eq 2;
	$new{ $n->[0][1] eq '*' ? 4
	    : $n->[1][0] eq '*' ? 2 : 0}=1 if $me eq 3;
	$new{ $n->[1][0] eq '*' ? 1
	    : $n->[2][1] eq '*' ? 3 : 0}=1 if $me eq 4;
        
	# MARCH
        $new{1} = 1 if $n->[0][0] eq 1 && $me ne '*' && $me ne '|';
        $new{2} = 1 if $n->[2][0] eq 2 && $me ne '*' && $me ne '|';
        $new{3} = 1 if $n->[2][2] eq 3 && $me ne '*';
        $new{4} = 1 if $n->[0][2] eq 4 && $me ne '*';

	# DEMON
	if ($me eq '|' || $me eq '_') {
		my $open =  ($n->[0][2] ne ' ' || $n->[2][2] ne ' ');
			 # && ($n->[0][0] eq ' ' || $n->[2][0] eq ' ');
		$new{$open?'_':'|'} = 1;
	}

	delete $new{0};
        return ' ' unless keys %new;
	return any keys %new;
}

$dfa->addrule(
        DFA::Cellular::ProcRule->new(\&next)
);

 my %represent = map { $_ => '.' } 1..4;
# my %represent = ( 1 => '\\', 2 => '/', 3 => '\\', 4 => '/' );

$dfa->mapout( sub{my @states = eigenstates($_[0]);
		    @states==0	? ' '
		  : @states>1 ? ($_[0] eq '_' ? '_' : '+')
		  : $represent{$_[0]}||$_[0]
		 });


while (1) {
	$dfa->step;
 	$dfa->show;
}
